import numpy as np
import casadi as ca


class ExplicitEulerIntegrator():

    def __init__(self, x: ca.MX, u: ca.MX, f: ca.MX, dt: float, num_steps: int = 1):

        self.f_continuous_fun = ca.Function('f', [x, u], [f])

        self.dt = dt
        self.num_steps = num_steps

        # setup the integrator
        h = dt/num_steps
        x_next = x

        for _ in range(num_steps):
            k = self.f_continuous_fun(x_next, u)
            x_next = x_next + h*k

        self.f_discrete_fun = ca.Function('f_discrete', [x, u], [x_next])


    def simulate(self, x0: np.ndarray, u0: np.ndarray):
        return self.f_discrete_fun(x0, u0).full()



class RK4Integrator():

    def __init__(self, x: ca.MX, u: ca.MX, f: ca.MX, dt: float, num_steps: int = 1):

        self.f_continuous_fun = ca.Function('f', [x, u], [f])

        self.dt = dt
        self.num_steps = num_steps

        h = dt/num_steps
        # TODO add your implementation here / part 2.1
        # x_next = ...

        # self.f_discrete_fun = ca.Function('f_discrete', [x, u], [x_next])


    def simulate(self, x0: np.ndarray, u0: np.ndarray):
        return self.f_discrete_fun(x0, u0).full()



class CVODESIntegrator():

    def __init__(self, x: ca.MX, u: ca.MX, f: ca.MX, dt: float):

        self.f_continuous_fun = ca.Function('f', [x, u], [f])

        self.dt = dt

        # TODO add your implementation here / part 2.2
        # ode =
        # t0 =

        # x_next = ca.integrator('F', 'cvodes', ode, t0, dt)(x0=x, p=u)['xf']

        # self.f_discrete_fun = ca.Function('f_discrete', [x, u], [x_next])


    def simulate(self, x0: np.ndarray, u0: np.ndarray):
        return self.f_discrete_fun(x0, u0).full()


